@@ -1,3 +1,5 @@
+require 'grape/router'
+
 module Grape
   # The API class is the primary entry point for creating Grape APIs. Users
   # should subclass this class in order to build an API.
@@ -13,9 +15,8 @@ module Grape
 
       # Clears all defined routes, endpoints, etc., on this API.
       def reset!
-        @route_set = Rack::Mount::RouteSet.new
-        @endpoints = []
-        @routes = nil
+        reset_endpoints!
+        reset_routes!
         reset_validations!
       end
 
@@ -44,25 +45,21 @@ module Grape
         instance.call(env)
       end
 
-      # Create a scope without affecting the URL.
-      #
-      # @param _name [Symbol] Purely placebo, just allows to name the scope to
-      # make the code more readable.
-      def scope(_name = nil, &block)
-        within_namespace do
-          nest(block)
-        end
-      end
-
       # (see #cascade?)
       def cascade(value = nil)
         if value.nil?
-          inheritable_setting.namespace_inheritable.keys.include?(:cascade) ? !!namespace_inheritable(:cascade) : true
+          inheritable_setting.namespace_inheritable.keys.include?(:cascade) ? !namespace_inheritable(:cascade).nil? : true
         else
           namespace_inheritable(:cascade, value)
         end
       end
 
+      # see Grape::Router#recognize_path
+      def recognize_path(path)
+        LOCK.synchronize { compile } unless instance
+        instance.router.recognize_path(path)
+      end
+
       protected
 
       def prepare_routes
@@ -91,33 +88,34 @@ module Grape
       def inherit_settings(other_settings)
         top_level_setting.inherit_from other_settings.point_in_time_copy
 
-        endpoints.each(&:reset_routes!)
-
-        @routes = nil
+        reset_routes!
       end
     end
 
+    attr_reader :router
+
     # Builds the routes from the defined endpoints, effectively compiling
     # this API into a usable form.
     def initialize
-      @route_set = Rack::Mount::RouteSet.new
+      @router = Router.new
       add_head_not_allowed_methods_and_options_methods
       self.class.endpoints.each do |endpoint|
-        endpoint.mount_in(@route_set)
+        endpoint.mount_in(@router)
       end
 
-      @route_set.freeze
+      @router.compile!
+      @router.freeze
     end
 
     # Handle a request. See Rack documentation for what `env` is.
     def call(env)
-      result = @route_set.call(env)
+      result = @router.call(env)
       result[1].delete(Grape::Http::Headers::X_CASCADE) unless cascade?
       result
     end
 
     # Some requests may return a HTTP 404 error if grape cannot find a matching
-    # route. In this case, Rack::Mount adds a X-Cascade header to the response
+    # route. In this case, Grape::Router adds a X-Cascade header to the response
     # and sets it to 'pass', indicating to grape's parents they should keep
     # looking for a matching route on other resources.
     #
@@ -139,13 +137,24 @@ module Grape
     # will return an HTTP 405 response for any HTTP method that the resource
     # cannot handle.
     def add_head_not_allowed_methods_and_options_methods
-      methods_per_path = {}
+      routes_map = {}
 
       self.class.endpoints.each do |endpoint|
         routes = endpoint.routes
         routes.each do |route|
-          methods_per_path[route.route_path] ||= []
-          methods_per_path[route.route_path] << route.route_method
+          # using the :any shorthand produces [nil] for route methods, substitute all manually
+          route_key = route.pattern.to_regexp
+          routes_map[route_key] ||= {}
+          route_settings = routes_map[route_key]
+          route_settings[:pattern] = route.pattern
+          route_settings[:requirements] = route.requirements
+          route_settings[:path] = route.origin
+          route_settings[:methods] ||= []
+          route_settings[:methods] << route.request_method
+          route_settings[:endpoint] = route.app
+
+          # using the :any shorthand produces [nil] for route methods, substitute all manually
+          route_settings[:methods] = %w(GET PUT POST DELETE PATCH HEAD OPTIONS) if route_settings[:methods].include?('ANY')
         end
       end
 
@@ -155,35 +164,48 @@ module Grape
       # informations again.
       without_root_prefix do
         without_versioning do
-          methods_per_path.each do |path, methods|
+          routes_map.each do |_, config|
+            methods = config[:methods]
+            path = config[:path]
             allowed_methods = methods.dup
+
             unless self.class.namespace_inheritable(:do_not_route_head)
               allowed_methods |= [Grape::Http::Headers::HEAD] if allowed_methods.include?(Grape::Http::Headers::GET)
             end
 
-            allow_header = ([Grape::Http::Headers::OPTIONS] | allowed_methods).join(', ')
+            allow_header = (self.class.namespace_inheritable(:do_not_route_options) ? allowed_methods : [Grape::Http::Headers::OPTIONS] | allowed_methods).join(', ')
+
             unless self.class.namespace_inheritable(:do_not_route_options)
-              unless allowed_methods.include?(Grape::Http::Headers::OPTIONS)
-                self.class.options(path, {}) do
-                  header 'Allow', allow_header
-                  status 204
-                  ''
-                end
-              end
+              generate_options_method(path, allow_header, config) unless allowed_methods.include?(Grape::Http::Headers::OPTIONS)
             end
 
-            not_allowed_methods = %w(GET PUT POST DELETE PATCH HEAD) - allowed_methods
-            not_allowed_methods << Grape::Http::Headers::OPTIONS if self.class.namespace_inheritable(:do_not_route_options)
-            self.class.route(not_allowed_methods, path) do
-              header 'Allow', allow_header
-              status 405
-              ''
-            end
+            attributes = config.merge(allowed_methods: allowed_methods, allow_header: allow_header)
+            generate_not_allowed_method(config[:pattern], attributes)
           end
         end
       end
     end
 
+    # Generate an 'OPTIONS' route for a pre-exisiting user defined route
+    def generate_options_method(path, allow_header, options = {})
+      self.class.options(path, options) do
+        header 'Allow', allow_header
+        status 204
+        ''
+      end
+    end
+
+    # Generate a route that returns an HTTP 405 response for a user defined
+    # path on methods not specified
+    def generate_not_allowed_method(pattern, attributes = {})
+      not_allowed_methods = %w(GET PUT POST DELETE PATCH HEAD) - attributes[:allowed_methods]
+      not_allowed_methods << Grape::Http::Headers::OPTIONS if self.class.namespace_inheritable(:do_not_route_options)
+
+      return if not_allowed_methods.empty?
+
+      @router.associate_routes(pattern, attributes.merge(not_allowed_methods: not_allowed_methods))
+    end
+
     # Allows definition of endpoints that ignore the versioning configuration
     # used by the rest of your API.
     def without_versioning(&_block)
