@@ -3,7 +3,7 @@ module Grape
   module Exceptions
     class UnknownParameter < Base
       def initialize(param)
-        super(message: compose_message('unknown_parameter', param: param))
+        super(message: compose_message(:unknown_parameter, param: param))
       end
     end
   end
