@@ -3,7 +3,7 @@ module Grape
   module Exceptions
     class UnsupportedGroupTypeError < Base
       def initialize
-        super(message: compose_message('unsupported_group_type'))
+        super(message: compose_message(:unsupported_group_type))
       end
     end
   end
