@@ -4,11 +4,12 @@ module Grape
   module Exceptions
     class Validation < Grape::Exceptions::Base
       attr_accessor :params
+      attr_accessor :message_key
 
       def initialize(args = {})
         fail 'Params are missing:' unless args.key? :params
         @params = args[:params]
-        args[:message] = translate_message(args[:message_key]) if args.key? :message_key
+        args[:message] = translate_message(args[:message]) if args.key? :message
         super
       end
 
