@@ -29,7 +29,7 @@ module Grape
     # Join the namespaces from a list of settings to create a path prefix.
     # @param settings [Array] list of Grape::Util::InheritableSettings.
     def self.joined_space_path(settings)
-      Rack::Mount::Utils.normalize_path(joined_space(settings))
+      Grape::Router.normalize_path(joined_space(settings))
     end
   end
 end
