@@ -2,7 +2,7 @@ module Grape
   # Represents a path to an endpoint.
   class Path
     def self.prepare(raw_path, namespace, settings)
-      Path.new(raw_path, namespace, settings).path_with_suffix
+      Path.new(raw_path, namespace, settings)
     end
 
     attr_reader :raw_path, :namespace, :settings
@@ -22,25 +22,25 @@ module Grape
     end
 
     def uses_specific_format?
-      !!(settings[:format] && settings[:content_types].size == 1)
+      !!(settings[:format] && Array(settings[:content_types]).size == 1)
     end
 
     def uses_path_versioning?
       !!(settings[:version] && settings[:version_options][:using] == :path)
     end
 
-    def has_namespace?
+    def namespace?
       namespace && namespace.to_s =~ /^\S/ && namespace != '/'
     end
 
-    def has_path?
+    def path?
       raw_path && raw_path.to_s =~ /^\S/ && raw_path != '/'
     end
 
     def suffix
       if uses_specific_format?
         "(.#{settings[:format]})"
-      elsif !uses_path_versioning? || (has_namespace? || has_path?)
+      elsif !uses_path_versioning? || (namespace? || path?)
         '(.:format)'
       else
         '(/.:format)'
@@ -48,7 +48,7 @@ module Grape
     end
 
     def path
-      Rack::Mount::Utils.normalize_path(parts.join('/'))
+      Grape::Router.normalize_path(parts.join('/'))
     end
 
     def path_with_suffix
