@@ -15,7 +15,7 @@ module Grape
 
         attrs = AttributesIterator.new(self, @scope, params)
         attrs.each do |resource_params, attr_name|
-          if resource_params[attr_name].nil?
+          if resource_params.is_a?(Hash) && resource_params[attr_name].nil?
             validate_param!(attr_name, resource_params)
           end
         end
