@@ -5,11 +5,20 @@ module Grape
       def validate!(params)
         super
         if scope_requires_params && none_of_restricted_params_is_present
-          fail Grape::Exceptions::Validation, params: all_keys, message_key: :exactly_one
+          fail Grape::Exceptions::Validation, params: all_keys, message: message(:exactly_one)
         end
         params
       end
 
+      def message(default_key = nil)
+        options = instance_variable_get(:@option)
+        if options_key?(:message)
+          (options_key?(default_key, options[:message]) ? options[:message][default_key] : options[:message])
+        else
+          default_key
+        end
+      end
+
       private
 
       def none_of_restricted_params_is_present
