@@ -7,9 +7,8 @@ module Grape
       end
 
       def validate_param!(attr_name, params)
-        unless params.respond_to?(:key?) && params.key?(attr_name)
-          fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message_key: :presence
-        end
+        return if params.respond_to?(:key?) && params.key?(attr_name)
+        fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message(:presence)
       end
     end
   end
