@@ -136,7 +136,7 @@ describe Grape::API do
     it 'adds the association to the :representations setting' do
       klass = Class.new
       subject.represent Object, with: klass
-      expect(Grape::DSL::Configuration.stacked_hash_to_hash(subject.namespace_stackable(:representations))[Object]).to eq(klass)
+      expect(subject.namespace_stackable_with_hash(:representations)[Object]).to eq(klass)
     end
   end
 
@@ -211,7 +211,7 @@ describe Grape::API do
     end
 
     %w(group resource resources segment).each do |als|
-      it '`.#{als}` is an alias' do
+      it "`.#{als}` is an alias" do
         inner_namespace = nil
         subject.send(als, :awesome) do
           inner_namespace = namespace
@@ -235,7 +235,7 @@ describe Grape::API do
       expect(last_response.body).to eq('23')
     end
 
-    it 'should be able to define requirements with a single hash' do
+    it 'defines requirements with a single hash' do
       subject.namespace :users do
         route_param :id, requirements: /[0-9]+/ do
           get do
@@ -249,6 +249,18 @@ describe Grape::API do
       get '/users/23'
       expect(last_response.status).to eq(200)
     end
+
+    context 'with param type definitions' do
+      it 'is used by passing to options' do
+        subject.namespace :route_param do
+          route_param :foo, type: Integer do
+            get { params.to_json }
+          end
+        end
+        get '/route_param/1234'
+        expect(last_response.body).to eq("{\"foo\":1234}")
+      end
+    end
   end
 
   describe '.route' do
@@ -399,7 +411,7 @@ describe Grape::API do
       end
 
       subject.endpoints.first.routes.each do |route|
-        expect(route.route_path).to eql '/abc(.:format)'
+        expect(route.path).to eql '/abc(.:format)'
       end
 
       get '/abc'
@@ -482,7 +494,7 @@ describe Grape::API do
 
     verbs = %w(post get head delete put options patch)
     verbs.each do |verb|
-      it 'allows and properly constrain a #{verb.upcase} method' do
+      it "allows and properly constrain a #{verb.upcase} method" do
         subject.send(verb, '/example') do
           verb
         end
@@ -511,10 +523,28 @@ describe Grape::API do
       end
       put '/example'
       expect(last_response.status).to eql 405
-      expect(last_response.body).to eql ''
+      expect(last_response.body).to eql '405 Not Allowed'
       expect(last_response.headers['X-Custom-Header']).to eql 'foo'
     end
 
+    context 'when format is xml' do
+      it 'returns a 405 for an unsupported method' do
+        subject.format :xml
+        subject.get 'example' do
+          'example'
+        end
+
+        put '/example'
+        expect(last_response.status).to eql 405
+        expect(last_response.body).to eq <<-XML
+<?xml version="1.0" encoding="UTF-8"?>
+<error>
+  <message>405 Not Allowed</message>
+</error>
+XML
+      end
+    end
+
     specify '405 responses includes an Allow header specifying supported methods' do
       subject.get 'example' do
         'example'
@@ -537,25 +567,90 @@ describe Grape::API do
       expect(last_response.headers['Content-Type']).to eql 'text/plain'
     end
 
-    it 'adds an OPTIONS route that returns a 204, an Allow header and a X-Custom-Header' do
-      subject.before { header 'X-Custom-Header', 'foo' }
-      subject.get 'example' do
-        'example'
+    describe 'adds an OPTIONS route that' do
+      before do
+        subject.before { header 'X-Custom-Header', 'foo' }
+        subject.get 'example' do
+          'example'
+        end
+        subject.route :any, '*path' do
+          error! :not_found, 404
+        end
+        options '/example'
+      end
+
+      it 'returns a 204' do
+        expect(last_response.status).to eql 204
+      end
+
+      it 'has an empty body' do
+        expect(last_response.body).to be_blank
+      end
+
+      it 'has an Allow header' do
+        expect(last_response.headers['Allow']).to eql 'OPTIONS, GET, HEAD'
+      end
+
+      it 'has a X-Custom-Header' do
+        expect(last_response.headers['X-Custom-Header']).to eql 'foo'
+      end
+
+      it 'has no Content-Type' do
+        expect(last_response.content_type).to be_nil
+      end
+
+      it 'has no Content-Length' do
+        expect(last_response.content_length).to be_nil
       end
-      options '/example'
-      expect(last_response.status).to eql 204
-      expect(last_response.body).to eql ''
-      expect(last_response.headers['Allow']).to eql 'OPTIONS, GET, HEAD'
-      expect(last_response.headers['X-Custom-Header']).to eql 'foo'
     end
 
-    it 'allows HEAD on a GET request' do
-      subject.get 'example' do
-        'example'
+    describe 'adds a 405 Not Allowed route that' do
+      before do
+        subject.before { header 'X-Custom-Header', 'foo' }
+        subject.post :example do
+          'example'
+        end
+        subject.route :any, '*path' do
+          error! :not_found, 404
+        end
+        get '/example'
+      end
+
+      it 'returns a 405' do
+        expect(last_response.status).to eql 405
+      end
+
+      it 'contains error message in body' do
+        expect(last_response.body).to eq '405 Not Allowed'
+      end
+
+      it 'has an Allow header' do
+        expect(last_response.headers['Allow']).to eql 'OPTIONS, POST'
+      end
+
+      it 'has a X-Custom-Header' do
+        expect(last_response.headers['X-Custom-Header']).to eql 'foo'
+      end
+    end
+
+    context 'allows HEAD on a GET request that' do
+      before do
+        subject.get 'example' do
+          'example'
+        end
+        subject.route :any, '*path' do
+          error! :not_found, 404
+        end
+        head '/example'
+      end
+
+      it 'returns a 200' do
+        expect(last_response.status).to eql 200
+      end
+
+      it 'has an empty body' do
+        expect(last_response.body).to eql ''
       end
-      head '/example'
-      expect(last_response.status).to eql 200
-      expect(last_response.body).to eql ''
     end
 
     it 'overwrites the default HEAD request' do
@@ -596,10 +691,17 @@ describe Grape::API do
         'example'
       end
     end
-    it 'options does not exist' do
+
+    it 'does not create an OPTIONS route' do
       options '/example'
       expect(last_response.status).to eql 405
     end
+
+    it 'does not include OPTIONS in Allow header' do
+      options '/example'
+      expect(last_response.status).to eql 405
+      expect(last_response.headers['Allow']).to eql 'GET, HEAD'
+    end
   end
 
   describe 'filters' do
@@ -640,7 +742,7 @@ describe Grape::API do
     end
 
     it 'adds a after_validation filter' do
-      subject.after_validation { @foo = "first #{params[:id] }:#{params[:id].class}"  }
+      subject.after_validation { @foo = "first #{params[:id]}:#{params[:id].class}" }
       subject.after_validation { @bar = 'second' }
       subject.params do
         requires :id, type: Integer
@@ -787,7 +889,7 @@ describe Grape::API do
 
     it 'returns raw data when content type binary' do
       image_filename = 'grape.png'
-      file = File.open(image_filename, 'rb') { |io| io.read }
+      file = File.open(image_filename, 'rb', &:read)
       subject.format :binary
       subject.get('/binary_file') { File.binread(image_filename) }
       get '/binary_file'
@@ -795,6 +897,37 @@ describe Grape::API do
       expect(last_response.body).to eq(file)
     end
 
+    it 'returns the content of the file with file' do
+      file_content = 'This is some file content'
+      test_file = Tempfile.new('test')
+      test_file.write file_content
+      test_file.rewind
+
+      subject.get('/file') { file test_file }
+      get '/file'
+      expect(last_response.headers['Content-Length']).to eq('25')
+      expect(last_response.headers['Content-Type']).to eq('text/plain')
+      expect(last_response.body).to eq(file_content)
+    end
+
+    it 'streams the content of the file with stream' do
+      test_stream = Enumerator.new do |blk|
+        blk.yield 'This is some'
+        blk.yield ' file content'
+      end
+
+      subject.use Rack::Chunked
+      subject.get('/stream') { stream test_stream }
+      get '/stream', {}, 'HTTP_VERSION' => 'HTTP/1.1'
+
+      expect(last_response.headers['Content-Type']).to eq('text/plain')
+      expect(last_response.headers['Content-Length']).to eq(nil)
+      expect(last_response.headers['Cache-Control']).to eq('no-cache')
+      expect(last_response.headers['Transfer-Encoding']).to eq('chunked')
+
+      expect(last_response.body).to eq("c\r\nThis is some\r\nd\r\n file content\r\n0\r\n\r\n")
+    end
+
     it 'sets content type for error' do
       subject.get('/error') { error!('error in plain text', 500) }
       get '/error'
@@ -817,6 +950,22 @@ describe Grape::API do
       expect(last_response.headers['Content-Type']).to eql 'application/xml'
     end
 
+    it 'includes extension in format' do
+      subject.get(':id') { params[:format] }
+
+      get '/baz.bar'
+      expect(last_response.status).to eq 200
+      expect(last_response.body).to eq 'bar'
+    end
+
+    it 'does not include extension in id' do
+      subject.format :json
+      subject.get(':id') { params }
+
+      get '/baz.bar'
+      expect(last_response.status).to eq 404
+    end
+
     context 'with a custom content_type' do
       before do
         subject.content_type :custom, 'application/custom'
@@ -843,7 +992,7 @@ describe Grape::API do
           filename = params[:file][:filename]
           content_type MIME::Types.type_for(filename)[0].to_s
           env['api.format'] = :binary # there's no formatter for :binary, data will be returned "as is"
-          header 'Content-Disposition', "attachment; filename*=UTF-8''#{URI.escape(filename)}"
+          header 'Content-Disposition', "attachment; filename*=UTF-8''#{CGI.escape(filename)}"
           params[:file][:tempfile].read
         end
       end
@@ -1035,7 +1184,7 @@ describe Grape::API do
 
       subject.get(:hello) { 'Hello, world.' }
       get '/hello', {}, 'HTTP_AUTHORIZATION' => encode_basic_auth('allow', 'whatever')
-      expect(basic_auth_context).to be_an_instance_of(Grape::Endpoint)
+      expect(basic_auth_context).to be_a_kind_of(Grape::Endpoint)
     end
 
     it 'has access to helper methods' do
@@ -1094,11 +1243,7 @@ describe Grape::API do
     it 'defaults to a standard logger log format' do
       t = Time.at(100)
       allow(Time).to receive(:now).and_return(t)
-      if ActiveSupport::VERSION::MAJOR >= 4
-        expect(subject.io).to receive(:write).with("I, [#{Logger::Formatter.new.send(:format_datetime, t)}\##{Process.pid}]  INFO -- : this will be logged\n")
-      else
-        expect(subject.io).to receive(:write).with("this will be logged\n")
-      end
+      expect(subject.io).to receive(:write).with("I, [#{Logger::Formatter.new.send(:format_datetime, t)}\##{Process.pid}]  INFO -- : this will be logged\n")
       subject.logger.info 'this will be logged'
     end
   end
@@ -1240,7 +1385,21 @@ describe Grape::API do
       subject.get '/exception' do
         fail 'rain!'
       end
-      expect { get '/exception' }.to raise_error
+      expect { get '/exception' }.to raise_error(RuntimeError, 'rain!')
+    end
+
+    it 'uses custom helpers defined by using #helpers method' do
+      subject.helpers do
+        def custom_error!(name)
+          error! "hello #{name}"
+        end
+      end
+      subject.rescue_from(ArgumentError) { custom_error! :bob }
+      subject.get '/custom_error' do
+        fail ArgumentError
+      end
+      get '/custom_error'
+      expect(last_response.body).to eq 'hello bob'
     end
 
     it 'rescues all errors if rescue_from :all is called' do
@@ -1273,26 +1432,28 @@ describe Grape::API do
       get '/rescued'
       expect(last_response.status).to eql 500
 
-      expect { get '/unrescued' }.to raise_error
+      expect { get '/unrescued' }.to raise_error(RuntimeError, 'beefcake')
     end
 
     context 'CustomError subclass of Grape::Exceptions::Base' do
       before do
-        class CustomError < Grape::Exceptions::Base; end
+        module ApiSpec
+          class CustomError < Grape::Exceptions::Base; end
+        end
       end
 
       it 'does not re-raise exceptions of type Grape::Exceptions::Base' do
-        subject.get('/custom_exception') { fail CustomError }
+        subject.get('/custom_exception') { fail ApiSpec::CustomError }
 
         expect { get '/custom_exception' }.not_to raise_error
       end
 
       it 'rescues custom grape exceptions' do
-        subject.rescue_from CustomError do |e|
+        subject.rescue_from ApiSpec::CustomError do |e|
           rack_response('New Error', e.status)
         end
         subject.get '/custom_error' do
-          fail CustomError, status: 400, message: 'Custom Error'
+          fail ApiSpec::CustomError, status: 400, message: 'Custom Error'
         end
 
         get '/custom_error'
@@ -1304,7 +1465,7 @@ describe Grape::API do
     it 'can rescue exceptions raised in the formatter' do
       formatter = double(:formatter)
       allow(formatter).to receive(:call) { fail StandardError }
-      allow(Grape::Formatter::Base).to receive(:formatter_for) { formatter }
+      allow(Grape::Formatter).to receive(:formatter_for) { formatter }
 
       subject.rescue_from :all do |_e|
         rack_response('Formatter Error', 500)
@@ -1404,7 +1565,7 @@ describe Grape::API do
 
   describe '.rescue_from klass, lambda' do
     it 'rescues an error with the lambda' do
-      subject.rescue_from ArgumentError, -> {
+      subject.rescue_from ArgumentError, lambda {
         rack_response('rescued with a lambda', 400)
       }
       subject.get('/rescue_lambda') { fail ArgumentError }
@@ -1415,7 +1576,7 @@ describe Grape::API do
     end
 
     it 'can execute the lambda with an argument' do
-      subject.rescue_from ArgumentError, ->(e) {
+      subject.rescue_from ArgumentError, lambda { |e|
         rack_response(e.message, 400)
       }
       subject.get('/rescue_lambda') { fail ArgumentError, 'lambda takes an argument' }
@@ -1426,38 +1587,48 @@ describe Grape::API do
     end
   end
 
-  describe '.rescue_from klass, with: method' do
-    it 'rescues an error with the specified message' do
-      def rescue_arg_error
-        Rack::Response.new('rescued with a method', 400)
+  describe '.rescue_from klass, with: :method_name' do
+    it 'rescues an error with the specified method name' do
+      subject.helpers do
+        def rescue_arg_error
+          error!('500 ArgumentError', 500)
+        end
       end
-
-      subject.rescue_from ArgumentError, with: rescue_arg_error
+      subject.rescue_from ArgumentError, with: :rescue_arg_error
       subject.get('/rescue_method') { fail ArgumentError }
 
       get '/rescue_method'
-      expect(last_response.status).to eq(400)
-      expect(last_response.body).to eq('rescued with a method')
+      expect(last_response.status).to eq(500)
+      expect(last_response.body).to eq('500 ArgumentError')
+    end
+
+    it 'aborts if the specified method name does not exist' do
+      subject.rescue_from :all, with: :not_exist_method
+      subject.get('/rescue_method') { fail StandardError }
+
+      expect { get '/rescue_method' }.to raise_error(NoMethodError, 'undefined method `not_exist_method\'')
     end
   end
 
   describe '.rescue_from klass, rescue_subclasses: boolean' do
     before do
-      module APIErrors
-        class ParentError < StandardError; end
-        class ChildError < ParentError; end
+      module ApiSpec
+        module APIErrors
+          class ParentError < StandardError; end
+          class ChildError < ParentError; end
+        end
       end
     end
 
     it 'rescues error as well as subclass errors with rescue_subclasses option set' do
-      subject.rescue_from APIErrors::ParentError, rescue_subclasses: true do |e|
+      subject.rescue_from ApiSpec::APIErrors::ParentError, rescue_subclasses: true do |e|
         rack_response("rescued from #{e.class.name}", 500)
       end
       subject.get '/caught_child' do
-        fail APIErrors::ChildError
+        fail ApiSpec::APIErrors::ChildError
       end
       subject.get '/caught_parent' do
-        fail APIErrors::ParentError
+        fail ApiSpec::APIErrors::ParentError
       end
       subject.get '/uncaught_parent' do
         fail StandardError
@@ -1471,11 +1642,11 @@ describe Grape::API do
     end
 
     it 'sets rescue_subclasses to true by default' do
-      subject.rescue_from APIErrors::ParentError do |e|
+      subject.rescue_from ApiSpec::APIErrors::ParentError do |e|
         rack_response("rescued from #{e.class.name}", 500)
       end
       subject.get '/caught_child' do
-        fail APIErrors::ChildError
+        fail ApiSpec::APIErrors::ChildError
       end
 
       get '/caught_child'
@@ -1483,13 +1654,13 @@ describe Grape::API do
     end
 
     it 'does not rescue child errors if rescue_subclasses is false' do
-      subject.rescue_from APIErrors::ParentError, rescue_subclasses: false do |e|
+      subject.rescue_from ApiSpec::APIErrors::ParentError, rescue_subclasses: false do |e|
         rack_response("rescued from #{e.class.name}", 500)
       end
       subject.get '/uncaught' do
-        fail APIErrors::ChildError
+        fail ApiSpec::APIErrors::ChildError
       end
-      expect { get '/uncaught' }.to raise_error(APIErrors::ChildError)
+      expect { get '/uncaught' }.to raise_error(ApiSpec::APIErrors::ChildError)
     end
   end
 
@@ -1541,15 +1712,17 @@ describe Grape::API do
 
     context 'class' do
       before :each do
-        class CustomErrorFormatter
-          def self.call(message, _backtrace, _options, _env)
-            "message: #{message} @backtrace"
+        module ApiSpec
+          class CustomErrorFormatter
+            def self.call(message, _backtrace, _options, _env)
+              "message: #{message} @backtrace"
+            end
           end
         end
       end
       it 'returns a custom error format' do
         subject.rescue_from :all, backtrace: true
-        subject.error_formatter :txt, CustomErrorFormatter
+        subject.error_formatter :txt, ApiSpec::CustomErrorFormatter
         subject.get '/exception' do
           fail 'rain!'
         end
@@ -1561,16 +1734,18 @@ describe Grape::API do
     describe 'with' do
       context 'class' do
         before :each do
-          class CustomErrorFormatter
-            def self.call(message, _backtrace, _option, _env)
-              "message: #{message} @backtrace"
+          module ApiSpec
+            class CustomErrorFormatter
+              def self.call(message, _backtrace, _option, _env)
+                "message: #{message} @backtrace"
+              end
             end
           end
         end
 
         it 'returns a custom error format' do
           subject.rescue_from :all, backtrace: true
-          subject.error_formatter :txt, with: CustomErrorFormatter
+          subject.error_formatter :txt, with: ApiSpec::CustomErrorFormatter
           subject.get('/exception') { fail 'rain!' }
 
           get '/exception'
@@ -1607,13 +1782,23 @@ describe Grape::API do
       get '/error'
       expect(last_response.body).to eql 'Access Denied'
     end
-    it 'rescues error! and return json' do
-      subject.format :json
-      subject.get '/error' do
-        error!('Access Denied', 401)
+    context 'with json format' do
+      before { subject.format :json }
+
+      it 'rescues error! called with a string and returns json' do
+        subject.get('/error') { error!(:failure, 401) }
+      end
+      it 'rescues error! called with a symbol and returns json' do
+        subject.get('/error') { error!(:failure, 401) }
+      end
+      it 'rescues error! called with a hash and returns json' do
+        subject.get('/error') { error!({ error: :failure }, 401) }
+      end
+
+      after do
+        get '/error'
+        expect(last_response.body).to eql('{"error":"failure"}')
       end
-      get '/error'
-      expect(last_response.body).to eql '{"error":"Access Denied"}'
     end
   end
 
@@ -1648,8 +1833,8 @@ describe Grape::API do
   describe '.formatter' do
     context 'multiple formatters' do
       before :each do
-        subject.formatter :json, ->(object, _env) { "{\"custom_formatter\":\"#{object[:some] }\"}" }
-        subject.formatter :txt, ->(object, _env) { "custom_formatter: #{object[:some] }" }
+        subject.formatter :json, ->(object, _env) { "{\"custom_formatter\":\"#{object[:some]}\"}" }
+        subject.formatter :txt, ->(object, _env) { "custom_formatter: #{object[:some]}" }
         subject.get :simple do
           { some: 'hash' }
         end
@@ -1667,7 +1852,7 @@ describe Grape::API do
       before :each do
         subject.content_type :json, 'application/json'
         subject.content_type :custom, 'application/custom'
-        subject.formatter :custom, ->(object, _env) { "{\"custom_formatter\":\"#{object[:some] }\"}" }
+        subject.formatter :custom, ->(object, _env) { "{\"custom_formatter\":\"#{object[:some]}\"}" }
         subject.get :simple do
           { some: 'hash' }
         end
@@ -1682,15 +1867,17 @@ describe Grape::API do
       end
     end
     context 'custom formatter class' do
-      module CustomFormatter
-        def self.call(object, _env)
-          "{\"custom_formatter\":\"#{object[:some] }\"}"
+      module ApiSpec
+        module CustomFormatter
+          def self.call(object, _env)
+            "{\"custom_formatter\":\"#{object[:some]}\"}"
+          end
         end
       end
       before :each do
         subject.content_type :json, 'application/json'
         subject.content_type :custom, 'application/custom'
-        subject.formatter :custom, CustomFormatter
+        subject.formatter :custom, ApiSpec::CustomFormatter
         subject.get :simple do
           { some: 'hash' }
         end
@@ -1734,15 +1921,17 @@ describe Grape::API do
       end
     end
     context 'custom parser class' do
-      module CustomParser
-        def self.call(object, _env)
-          { object.to_sym => object.to_s.reverse }
+      module ApiSpec
+        module CustomParser
+          def self.call(object, _env)
+            { object.to_sym => object.to_s.reverse }
+          end
         end
       end
       before :each do
         subject.content_type :txt, 'text/plain'
         subject.content_type :custom, 'text/custom'
-        subject.parser :custom, CustomParser
+        subject.parser :custom, ApiSpec::CustomParser
         subject.put :simple do
           params[:simple]
         end
@@ -1767,7 +1956,7 @@ describe Grape::API do
       before :each do
         subject.parser :json, nil
         subject.put 'data' do
-          "body: #{env['api.request.body'] }"
+          "body: #{env['api.request.body']}"
         end
       end
       it 'does not parse data' do
@@ -1857,7 +2046,7 @@ describe Grape::API do
     end
 
     it 'presented with' do
-      error = { code: 408, with: error_presenter }
+      error = { code: 408, with: error_presenter }.freeze
       subject.get '/exception' do
         error! error, 408
       end
@@ -1883,9 +2072,9 @@ describe Grape::API do
       it 'returns one route' do
         expect(subject.routes.size).to eq(1)
         route = subject.routes[0]
-        expect(route.route_version).to be_nil
-        expect(route.route_path).to eq('/ping(.:format)')
-        expect(route.route_method).to eq('GET')
+        expect(route.version).to be_nil
+        expect(route.path).to eq('/ping(.:format)')
+        expect(route.request_method).to eq('GET')
       end
     end
     describe 'api structure with two versions and a namespace' do
@@ -1913,18 +2102,18 @@ describe Grape::API do
       end
       it 'sets route paths' do
         expect(subject.routes.size).to be >= 2
-        expect(subject.routes[0].route_path).to eq('/:version/version(.:format)')
-        expect(subject.routes[1].route_path).to eq('/p/:version/n1/n2/version(.:format)')
+        expect(subject.routes[0].path).to eq('/:version/version(.:format)')
+        expect(subject.routes[1].path).to eq('/p/:version/n1/n2/version(.:format)')
       end
       it 'sets route versions' do
-        expect(subject.routes[0].route_version).to eq('v1')
-        expect(subject.routes[1].route_version).to eq('v2')
+        expect(subject.routes[0].version).to eq('v1')
+        expect(subject.routes[1].version).to eq('v2')
       end
       it 'sets a nested namespace' do
-        expect(subject.routes[1].route_namespace).to eq('/n1/n2')
+        expect(subject.routes[1].namespace).to eq('/n1/n2')
       end
       it 'sets prefix' do
-        expect(subject.routes[1].route_prefix).to eq('p')
+        expect(subject.routes[1].prefix).to eq('p')
       end
     end
     describe 'api structure with additional parameters' do
@@ -1945,9 +2134,9 @@ describe Grape::API do
         get '/split/a,b,c.json', token: ',', limit: '2'
         expect(last_response.body).to eq('["a","b,c"]')
       end
-      it 'sets route_params' do
+      it 'sets params' do
         expect(subject.routes.map { |route|
-          { params: route.route_params }
+          { params: route.params }
         }).to eq [
           {
             params: {
@@ -1975,9 +2164,9 @@ describe Grape::API do
         subject.get 'two' do
         end
       end
-      it 'sets route_params' do
+      it 'sets params' do
         expect(subject.routes.map { |route|
-          { params: route.route_params }
+          { params: route.params }
         }).to eq [
           {
             params: {
@@ -2006,9 +2195,9 @@ describe Grape::API do
         subject.get 'two' do
         end
       end
-      it 'sets route_params' do
+      it 'sets params' do
         expect(subject.routes.map { |route|
-          { params: route.route_params }
+          { params: route.params }
         }).to eq [
           {
             params: {
@@ -2030,7 +2219,7 @@ describe Grape::API do
       it 'exposed' do
         expect(subject.routes.count).to eq 1
         route = subject.routes.first
-        expect(route.route_settings[:custom]).to eq(key: 'value')
+        expect(route.settings[:custom]).to eq(key: 'value')
       end
     end
     describe 'status' do
@@ -2064,9 +2253,17 @@ describe Grape::API do
       subject.get :first do; end
       expect(subject.routes.length).to eq(1)
       route = subject.routes.first
-      expect(route.route_description).to eq('first method')
+      expect(route.description).to eq('first method')
       expect(route.route_foo).to be_nil
-      expect(route.route_params).to eq({})
+      expect(route.params).to eq({})
+      expect(route.options).to be_a_kind_of(Hash)
+    end
+    it 'has params which does not include format and version as named captures' do
+      subject.version :v1, using: :path
+      subject.get :first do; end
+      param_keys = subject.routes.first.params.keys
+      expect(param_keys).not_to include('format')
+      expect(param_keys).not_to include('version')
     end
     it 'describes methods separately' do
       subject.desc 'first method'
@@ -2075,7 +2272,7 @@ describe Grape::API do
       subject.get :second do; end
       expect(subject.routes.count).to eq(2)
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: 'first method', params: {} },
         { description: 'second method', params: {} }
@@ -2086,7 +2283,7 @@ describe Grape::API do
       subject.get :first do; end
       subject.get :second do; end
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: 'first method', params: {} },
         { description: nil, params: {} }
@@ -2098,7 +2295,7 @@ describe Grape::API do
         get 'second' do; end
       end
       expect(subject.routes.map { |route|
-        { description: route.route_description, foo: route.route_foo, params: route.route_params }
+        { description: route.description, foo: route.route_foo, params: route.params }
       }).to eq [
         { description: 'ns second', foo: 'bar', params: {} }
       ]
@@ -2107,7 +2304,7 @@ describe Grape::API do
       subject.desc 'method', details: 'method details'
       subject.get 'method' do; end
       expect(subject.routes.map { |route|
-        { description: route.route_description, details: route.route_details, params: route.route_params }
+        { description: route.description, details: route.details, params: route.params }
       }).to eq [
         { description: 'method', details: 'method details', params: {} }
       ]
@@ -2118,11 +2315,43 @@ describe Grape::API do
         params[:s].reverse
       end
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: 'Reverses a string.', params: { 's' => { desc: 'string to reverse', type: 'string' } } }
       ]
     end
+    it 'does not inherit param descriptions in consequent namespaces' do
+      subject.desc 'global description'
+      subject.params do
+        requires :param1
+        optional :param2
+      end
+      subject.namespace 'ns1' do
+        get do; end
+      end
+      subject.params do
+        optional :param2
+      end
+      subject.namespace 'ns2' do
+        get do; end
+      end
+      routes_doc = subject.routes.map { |route|
+        { description: route.description, params: route.params }
+      }
+      expect(routes_doc).to eq [
+        { description: 'global description',
+          params: {
+            'param1' => { required: true },
+            'param2' => { required: false }
+          }
+        },
+        { description: 'global description',
+          params: {
+            'param2' => { required: false }
+          }
+        }
+      ]
+    end
     it 'merges the parameters of the namespace with the parameters of the method' do
       subject.desc 'namespace'
       subject.params do
@@ -2137,7 +2366,7 @@ describe Grape::API do
       end
 
       routes_doc = subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }
       expect(routes_doc).to eq [
         { description: 'method',
@@ -2169,7 +2398,7 @@ describe Grape::API do
         end
       end
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: 'method',
           params: {
@@ -2195,7 +2424,7 @@ describe Grape::API do
       end
       subject.get 'method' do; end
 
-      expect(subject.routes.map(&:route_params)).to eq [{
+      expect(subject.routes.map(&:params)).to eq [{
         'group1'         => { required: true, type: 'Array' },
         'group1[param1]' => { required: false, desc: 'group1 param1 desc' },
         'group1[param2]' => { required: true, desc: 'group1 param2 desc' },
@@ -2214,7 +2443,7 @@ describe Grape::API do
       end
       subject.get 'method' do; end
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: 'nesting',
           params: {
@@ -2238,7 +2467,7 @@ describe Grape::API do
       end
       subject.get 'method' do; end
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: nil, params: { 'one_param' => { required: true, desc: 'one param' } } }
       ]
@@ -2249,7 +2478,7 @@ describe Grape::API do
         params[:s].reverse
       end
       expect(subject.routes.map { |route|
-        { description: route.route_description, params: route.route_params }
+        { description: route.description, params: route.params }
       }).to eq [
         { description: 'Reverses a string.', params: { 's' => { desc: 'string to reverse', type: 'string' } } }
       ]
@@ -2357,8 +2586,8 @@ describe Grape::API do
           mount app
         end
         expect(subject.routes.size).to eq(2)
-        expect(subject.routes.first.route_path).to match(%r{\/cool\/awesome})
-        expect(subject.routes.last.route_path).to match(%r{\/cool\/sauce})
+        expect(subject.routes.first.path).to match(%r{\/cool\/awesome})
+        expect(subject.routes.last.path).to match(%r{\/cool\/sauce})
       end
 
       it 'mounts on a path' do
@@ -2577,10 +2806,10 @@ describe Grape::API do
     context 'plain' do
       before(:each) do
         subject.get '/' do
-          route.route_path
+          route.path
         end
         subject.get '/path' do
-          route.route_path
+          route.path
         end
       end
       it 'provides access to route info' do
@@ -2594,11 +2823,11 @@ describe Grape::API do
       before(:each) do
         subject.desc 'returns description'
         subject.get '/description' do
-          route.route_description
+          route.description
         end
         subject.desc 'returns parameters', params: { 'x' => 'y' }
         subject.get '/params/:id' do
-          route.route_params[params[:id]]
+          route.params[params[:id]]
         end
       end
       it 'returns route description' do
@@ -2749,10 +2978,10 @@ XML
       end
       it 'hash' do
         subject.get '/example' do
-          ActiveSupport::OrderedHash[
-            :example1, 'example1',
-            :example2, 'example2'
-        ]
+          {
+            example1: 'example1',
+            example2: 'example2'
+          }
         end
         get '/example'
         expect(last_response.status).to eq(200)
@@ -2817,7 +3046,7 @@ XML
     [true, false].each do |anchor|
       it "anchor=#{anchor}" do
         subject.route :any, '*path', anchor: anchor do
-          error!("Unrecognized request path: #{params[:path] } - #{env['PATH_INFO'] }#{env['SCRIPT_NAME'] }", 404)
+          error!("Unrecognized request path: #{params[:path]} - #{env['PATH_INFO']}#{env['SCRIPT_NAME']}", 404)
         end
         get '/v1/hello'
         expect(last_response.status).to eq(200)
@@ -2825,6 +3054,12 @@ XML
         get '/v2/hello'
         expect(last_response.status).to eq(200)
         expect(last_response.body).to eq('v2')
+        options '/v2/hello'
+        expect(last_response.status).to eq(204)
+        expect(last_response.body).to be_blank
+        head '/v2/hello'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to be_blank
         get '/foobar'
         expect(last_response.status).to eq(404)
         expect(last_response.body).to eq('Unrecognized request path: foobar - /foobar')
