@@ -10,19 +10,19 @@ describe Grape::Endpoint do
   describe '.before_each' do
     after { Grape::Endpoint.before_each(nil) }
 
-    it 'should be settable via block' do
-      block = lambda { |_endpoint| 'noop' }
+    it 'is settable via block' do
+      block = ->(_endpoint) { 'noop' }
       Grape::Endpoint.before_each(&block)
-      expect(Grape::Endpoint.before_each).to eq(block)
+      expect(Grape::Endpoint.before_each.first).to eq(block)
     end
 
-    it 'should be settable via reference' do
-      block = lambda { |_endpoint| 'noop' }
+    it 'is settable via reference' do
+      block = ->(_endpoint) { 'noop' }
       Grape::Endpoint.before_each block
-      expect(Grape::Endpoint.before_each).to eq(block)
+      expect(Grape::Endpoint.before_each.first).to eq(block)
     end
 
-    it 'should be able to override a helper' do
+    it 'is able to override a helper' do
       subject.get('/') { current_user }
       expect { get '/' }.to raise_error(NameError)
 
@@ -36,6 +36,28 @@ describe Grape::Endpoint do
       Grape::Endpoint.before_each(nil)
       expect { get '/' }.to raise_error(NameError)
     end
+
+    it 'is able to stack helper' do
+      subject.get('/') do
+        authenticate_user!
+        current_user
+      end
+      expect { get '/' }.to raise_error(NameError)
+
+      Grape::Endpoint.before_each do |endpoint|
+        allow(endpoint).to receive(:current_user).and_return('Bob')
+      end
+
+      Grape::Endpoint.before_each do |endpoint|
+        allow(endpoint).to receive(:authenticate_user!).and_return(true)
+      end
+
+      get '/'
+      expect(last_response.body).to eq('Bob')
+
+      Grape::Endpoint.before_each(nil)
+      expect { get '/' }.to raise_error(NameError)
+    end
   end
 
   describe '#initialize' do
@@ -239,6 +261,16 @@ describe Grape::Endpoint do
       end
     end
 
+    it 'does not work in a before filter' do
+      subject.before do
+        declared(params)
+      end
+      subject.get('/declared') { declared(params) }
+
+      expect { get('/declared') }.to raise_error(
+        Grape::DSL::InsideRoute::MethodNotYetAvailable)
+    end
+
     it 'has as many keys as there are declared params' do
       inner_params = nil
       subject.get '/declared' do
@@ -710,6 +742,16 @@ describe Grape::Endpoint do
       expect(last_response.body).to eq('{"dude":"rad"}')
     end
 
+    it 'accepts a frozen object' do
+      subject.get '/hey' do
+        error!({ 'dude' => 'rad' }.freeze, 403)
+      end
+
+      get '/hey.json'
+      expect(last_response.status).to eq(403)
+      expect(last_response.body).to eq('{"dude":"rad"}')
+    end
+
     it 'can specifiy headers' do
       subject.get '/hey' do
         error!({ 'dude' => 'rad' }, 403, 'X-Custom' => 'value')
@@ -742,7 +784,7 @@ describe Grape::Endpoint do
       get '/hey'
       expect(last_response.status).to eq 302
       expect(last_response.headers['Location']).to eq '/ha'
-      expect(last_response.body).to eq ''
+      expect(last_response.body).to eq 'This resource has been moved temporarily to /ha.'
     end
 
     it 'has status code 303 if it is not get request and it is http 1.1' do
@@ -752,6 +794,7 @@ describe Grape::Endpoint do
       post '/hey', {}, 'HTTP_VERSION' => 'HTTP/1.1'
       expect(last_response.status).to eq 303
       expect(last_response.headers['Location']).to eq '/ha'
+      expect(last_response.body).to eq 'An alternate resource is located at /ha.'
     end
 
     it 'support permanent redirect' do
@@ -761,7 +804,15 @@ describe Grape::Endpoint do
       get '/hey'
       expect(last_response.status).to eq 301
       expect(last_response.headers['Location']).to eq '/ha'
-      expect(last_response.body).to eq ''
+      expect(last_response.body).to eq 'This resource has been moved permanently to /ha.'
+    end
+
+    it 'allows for an optional redirect body override' do
+      subject.get('/hey') do
+        redirect '/ha', body: 'test body'
+      end
+      get '/hey'
+      expect(last_response.body).to eq 'test body'
     end
   end
 
@@ -846,13 +897,64 @@ describe Grape::Endpoint do
         expect(last_response.body).to eq('body')
       end
     end
+
+    it 'allows adding to response with present' do
+      subject.format :json
+      subject.before { present :before, 'before' }
+      subject.before_validation { present :before_validation, 'before_validation' }
+      subject.after_validation { present :after_validation, 'after_validation' }
+      subject.after { present :after, 'after' }
+      subject.get :all_filters do
+        present :endpoint, 'endpoint'
+      end
+
+      get '/all_filters'
+      json = JSON.parse(last_response.body)
+      expect(json.keys).to match_array %w(before before_validation after_validation endpoint after)
+    end
+
+    context 'when terminating the response with error!' do
+      it 'breaks normal call chain' do
+        called = []
+        subject.before { called << 'before' }
+        subject.before_validation { called << 'before_validation' }
+        subject.after_validation { error! :oops, 500 }
+        subject.after { called << 'after' }
+        subject.get :error_filters do
+          called << 'endpoint'
+          ''
+        end
+
+        get '/error_filters'
+        expect(last_response.status).to eql 500
+        expect(called).to match_array %w(before before_validation)
+      end
+
+      it 'allows prior and parent filters of same type to run' do
+        called = []
+        subject.before { called << 'parent' }
+        subject.namespace :parent do
+          before { called << 'prior' }
+          before { error! :oops, 500 }
+          before { called << 'subsequent' }
+          get :hello do
+            called << :endpoint
+            'Hello!'
+          end
+        end
+
+        get '/parent/hello'
+        expect(last_response.status).to eql 500
+        expect(called).to match_array %w(parent prior)
+      end
+    end
   end
 
   context 'anchoring' do
     verbs = %w(post get head delete put options patch)
 
     verbs.each do |verb|
-      it 'allows for the anchoring option with a #{verb.upcase} method' do
+      it "allows for the anchoring option with a #{verb.upcase} method" do
         subject.send(verb, '/example', anchor: true) do
           verb
         end
@@ -860,7 +962,7 @@ describe Grape::Endpoint do
         expect(last_response.status).to eql 404
       end
 
-      it 'anchors paths by default for the #{verb.upcase} method' do
+      it "anchors paths by default for the #{verb.upcase} method" do
         subject.send(verb, '/example') do
           verb
         end
@@ -868,7 +970,7 @@ describe Grape::Endpoint do
         expect(last_response.status).to eql 404
       end
 
-      it 'responds to /example/and/some/more for the non-anchored #{verb.upcase} method' do
+      it "responds to /example/and/some/more for the non-anchored #{verb.upcase} method" do
         subject.send(verb, '/example', anchor: false) do
           verb
         end
@@ -880,7 +982,7 @@ describe Grape::Endpoint do
   end
 
   context 'request' do
-    it 'should be set to the url requested' do
+    it 'is set to the url requested' do
       subject.get('/url') do
         request.url
       end
@@ -888,7 +990,7 @@ describe Grape::Endpoint do
       expect(last_response.body).to eq('http://example.org/url')
     end
     ['v1', :v1].each do |version|
-      it 'should include version #{version}' do
+      it "should include version #{version}" do
         subject.version version, using: :path
         subject.get('/url') do
           request.url
@@ -987,38 +1089,38 @@ describe Grape::Endpoint do
 
       # In order that the events finalized (time each block ended)
       expect(@events).to contain_exactly(
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: a_collection_containing_exactly(an_instance_of(Proc)),
                                                                        type: :before }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :before_validation }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after_validation }),
-        have_attributes(name: 'endpoint_render.grape',      payload: { endpoint: an_instance_of(Grape::Endpoint) }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_render.grape',      payload: { endpoint: a_kind_of(Grape::Endpoint) }),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after }),
-        have_attributes(name: 'endpoint_run.grape',         payload: { endpoint: an_instance_of(Grape::Endpoint),
-                                                                       env: an_instance_of(Hash) })
+        have_attributes(name: 'endpoint_run.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
+                                                               env: an_instance_of(Hash) })
       )
 
       # In order that events were initialized
       expect(@events.sort_by(&:time)).to contain_exactly(
-        have_attributes(name: 'endpoint_run.grape',         payload: { endpoint: an_instance_of(Grape::Endpoint),
-                                                                       env: an_instance_of(Hash) }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_run.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
+                                                               env: an_instance_of(Hash) }),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: a_collection_containing_exactly(an_instance_of(Proc)),
                                                                        type: :before }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :before_validation }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after_validation }),
-        have_attributes(name: 'endpoint_render.grape',      payload: { endpoint: an_instance_of(Grape::Endpoint) }),
-        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: an_instance_of(Grape::Endpoint),
+        have_attributes(name: 'endpoint_render.grape',      payload: { endpoint: a_kind_of(Grape::Endpoint) }),
+        have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after })
       )
