@@ -20,8 +20,9 @@ describe Rack do
       env = Rack::MockRequest.env_for('/', options)
 
       unless RUBY_PLATFORM == 'java'
-        major, minor, release = Rack.release.split('.').map(&:to_i)
-        pending 'Rack 1.5.3 or 1.6.1 required' unless major >= 1 && ((minor == 5 && release >= 3) || (minor >= 6))
+        major, minor, patch = Rack.release.split('.').map(&:to_i)
+        patch ||= 0 # rack <= 1.5.2 does not specify patch version
+        pending 'Rack 1.5.3 or 1.6.1 required' unless major >= 1 && ((minor == 5 && patch >= 3) || (minor >= 6))
       end
 
       expect(JSON.parse(app.call(env)[2].body.first)['params_keys']).to match_array('test')
