@@ -50,7 +50,7 @@ describe Grape::Middleware::Formatter do
   context 'error handling' do
     let(:formatter) { double(:formatter) }
     before do
-      allow(Grape::Formatter::Base).to receive(:formatter_for) { formatter }
+      allow(Grape::Formatter).to receive(:formatter_for) { formatter }
     end
 
     it 'rescues formatter-specific exceptions' do
@@ -66,7 +66,7 @@ describe Grape::Middleware::Formatter do
 
       expect do
         catch(:error) { subject.call('PATH_INFO' => '/somewhere.xml', 'HTTP_ACCEPT' => 'application/json') }
-      end.to raise_error
+      end.to raise_error(StandardError)
     end
   end
 
@@ -132,6 +132,18 @@ describe Grape::Middleware::Formatter do
       expect(subject.env['api.format']).to eq(:xml)
     end
 
+    context 'with custom vendored content types' do
+      before do
+        subject.options[:content_types] = {}
+        subject.options[:content_types][:custom] = 'application/vnd.test+json'
+      end
+
+      it 'it uses the custom type' do
+        subject.call('PATH_INFO' => '/info', 'HTTP_ACCEPT' => 'application/vnd.test+json')
+        expect(subject.env['api.format']).to eq(:custom)
+      end
+    end
+
     it 'parses headers with symbols as hash keys' do
       subject.call('PATH_INFO' => '/info', 'http_accept' => 'application/xml', system_time: '091293')
       expect(subject.env[:system_time]).to eq('091293')
@@ -157,6 +169,16 @@ describe Grape::Middleware::Formatter do
       _, headers, = subject.call('PATH_INFO' => '/info.custom')
       expect(headers['Content-type']).to eq('application/x-custom')
     end
+    it 'is set for vendored with registered type' do
+      subject.options[:content_types] = {}
+      subject.options[:content_types][:custom] = 'application/vnd.test+json'
+      _, headers, = subject.call('PATH_INFO' => '/info', 'HTTP_ACCEPT' => 'application/vnd.test+json')
+      expect(headers['Content-type']).to eq('application/vnd.test+json')
+    end
+    it 'is set to closest generic for custom vendored/versioned without registered type' do
+      _, headers, = subject.call('PATH_INFO' => '/info', 'HTTP_ACCEPT' => 'application/vnd.test+json')
+      expect(headers['Content-type']).to eq('application/json')
+    end
   end
 
   context 'format' do
@@ -179,11 +201,23 @@ describe Grape::Middleware::Formatter do
     end
   end
 
+  context 'no content responses' do
+    let(:no_content_response) { ->(status) { [status, {}, ['']] } }
+
+    Rack::Utils::STATUS_WITH_NO_ENTITY_BODY.each do |status|
+      it "does not modify a #{status} response" do
+        expected_response = no_content_response[status]
+        allow(app).to receive(:call).and_return(expected_response)
+        expect(subject.call({})).to eq(expected_response)
+      end
+    end
+  end
+
   context 'input' do
     %w(POST PATCH PUT DELETE).each do |method|
       ['application/json', 'application/json; charset=utf-8'].each do |content_type|
         context content_type do
-          it 'parses the body from #{method} and copies values into rack.request.form_hash' do
+          it "parses the body from #{method} and copies values into rack.request.form_hash" do
             io = StringIO.new('{"is_boolean":true,"string":"thing"}')
             subject.call(
               'PATH_INFO' => '/info',
@@ -222,7 +256,7 @@ describe Grape::Middleware::Formatter do
         expect(subject.env['rack.request.form_hash']['is_boolean']).to be true
         expect(subject.env['rack.request.form_hash']['string']).to eq('thing')
       end
-      it 'parses the body from an xml #{method} and copies values into rack.request.from_hash' do
+      it "parses the body from an xml #{method} and copies values into rack.request.from_hash" do
         io = StringIO.new('<thing><name>Test</name></thing>')
         subject.call(
           'PATH_INFO' => '/info.xml',
@@ -248,4 +282,33 @@ describe Grape::Middleware::Formatter do
       end
     end
   end
+
+  context 'send file' do
+    let(:app) { ->(_env) { [200, {}, @body] } }
+
+    it 'returns Grape::Uril::SendFileReponse' do
+      @body = Grape::ServeFile::FileResponse.new('file')
+      env = { 'PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/json' }
+      expect(subject.call(env)).to be_a(Grape::ServeFile::SendfileResponse)
+    end
+  end
+
+  context 'inheritable formatters' do
+    class InvalidFormatter
+      def self.call(_, _)
+        { message: 'invalid' }.to_json
+      end
+    end
+    let(:app) { ->(_env) { [200, {}, ['']] } }
+    before do
+      Grape::Formatter.register :invalid, InvalidFormatter
+      Grape::ContentTypes::CONTENT_TYPES[:invalid] = 'application/x-invalid'
+    end
+
+    it 'returns response by invalid formatter' do
+      env = { 'PATH_INFO' => '/hello.invalid', 'HTTP_ACCEPT' => 'application/x-invalid' }
+      _, _, bodies = *subject.call(env)
+      expect(bodies.body.first).to eq({ message: 'invalid' }.to_json)
+    end
+  end
 end
