@@ -50,19 +50,19 @@ describe Grape::Middleware::Versioner::Header do
     end
 
     ['v1', :v1].each do |version|
-      context 'when version is set to #{version{ ' do
+      context "when version is set to #{version}" do
         before do
           @options[:versions] = [version]
         end
 
         it 'is set' do
-          status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json')
+          status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json')
           expect(env['api.format']).to eql 'json'
           expect(status).to eq(200)
         end
 
         it 'is nil if not provided' do
-          status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1')
+          status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1')
           expect(env['api.format']).to eql nil
           expect(status).to eq(200)
         end
@@ -72,13 +72,13 @@ describe Grape::Middleware::Versioner::Header do
 
   context 'api.vendor' do
     it 'is set' do
-      status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor')
+      status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor')
       expect(env['api.vendor']).to eql 'vendor'
       expect(status).to eq(200)
     end
 
     it 'is set if format provided' do
-      status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor+json')
+      status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor+json')
       expect(env['api.vendor']).to eql 'vendor'
       expect(status).to eq(200)
     end
@@ -89,7 +89,7 @@ describe Grape::Middleware::Versioner::Header do
           expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
           expect(exception.headers).to eql('X-Cascade' => 'pass')
           expect(exception.status).to eql 406
-          expect(exception.message).to include 'API vendor or version not found'
+          expect(exception.message).to include 'API vendor not found'
         end
     end
 
@@ -99,13 +99,13 @@ describe Grape::Middleware::Versioner::Header do
       end
 
       it 'is set' do
-        status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1')
+        status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1')
         expect(env['api.vendor']).to eql 'vendor'
         expect(status).to eq(200)
       end
 
       it 'is set if format provided' do
-        status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json')
+        status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json')
         expect(env['api.vendor']).to eql 'vendor'
         expect(status).to eq(200)
       end
@@ -116,7 +116,7 @@ describe Grape::Middleware::Versioner::Header do
             expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
             expect(exception.headers).to eql('X-Cascade' => 'pass')
             expect(exception.status).to eql 406
-            expect(exception.message).to include('API vendor or version not found')
+            expect(exception.message).to include('API vendor not found')
           end
       end
     end
@@ -128,23 +128,23 @@ describe Grape::Middleware::Versioner::Header do
     end
 
     it 'is set' do
-      status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1')
+      status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1')
       expect(env['api.version']).to eql 'v1'
       expect(status).to eq(200)
     end
 
     it 'is set if format provided' do
-      status, _, env =  subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json')
+      status, _, env = subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json')
       expect(env['api.version']).to eql 'v1'
       expect(status).to eq(200)
     end
 
     it 'fails with 406 Not Acceptable if version is invalid' do
       expect { subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v2+json').last }.to raise_exception do |exception|
-        expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
+        expect(exception).to be_a(Grape::Exceptions::InvalidVersionHeader)
         expect(exception.headers).to eql('X-Cascade' => 'pass')
         expect(exception.status).to eql 406
-        expect(exception.message).to include('API vendor or version not found')
+        expect(exception.message).to include('API version not found')
       end
     end
   end
@@ -184,30 +184,12 @@ describe Grape::Middleware::Versioner::Header do
       end
     end
 
-    it 'fails with 406 Not Acceptable if type is a range' do
-      expect { subject.call('HTTP_ACCEPT' => '*/*').last }.to raise_exception do |exception|
-        expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
-        expect(exception.headers).to eql('X-Cascade' => 'pass')
-        expect(exception.status).to eql 406
-        expect(exception.message).to include('Accept header must not contain ranges ("*").')
-      end
-    end
-
-    it 'fails with 406 Not Acceptable if subtype is a range' do
-      expect { subject.call('HTTP_ACCEPT' => 'application/*').last }.to raise_exception do |exception|
-        expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
-        expect(exception.headers).to eql('X-Cascade' => 'pass')
-        expect(exception.status).to eql 406
-        expect(exception.message).to include('Accept header must not contain ranges ("*").')
-      end
-    end
-
     it 'succeeds if proper header is set' do
       expect(subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v1+json').first).to eq(200)
     end
   end
 
-  context 'when :strict and :cascade=>false are set' do
+  context 'when :strict and cascade: false' do
     before do
       @options[:versions] = ['v1']
       @options[:version_options][:strict] = true
@@ -223,6 +205,16 @@ describe Grape::Middleware::Versioner::Header do
       end
     end
 
+    it 'fails with 406 Not Acceptable if header is application/xml' do
+      expect { subject.call('HTTP_ACCEPT' => 'application/xml').last }
+        .to raise_exception do |exception|
+        expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
+        expect(exception.headers).to eql({})
+        expect(exception.status).to eql 406
+        expect(exception.message).to include('API vendor or version not found.')
+      end
+    end
+
     it 'fails with 406 Not Acceptable if header is empty' do
       expect { subject.call('HTTP_ACCEPT' => '').last }.to raise_exception do |exception|
         expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
@@ -232,21 +224,13 @@ describe Grape::Middleware::Versioner::Header do
       end
     end
 
-    it 'fails with 406 Not Acceptable if type is a range' do
-      expect { subject.call('HTTP_ACCEPT' => '*/*').last }.to raise_exception do |exception|
+    it 'fails with 406 Not Acceptable if header contains a single invalid accept' do
+      expect { subject.call('HTTP_ACCEPT' => 'application/json;application/vnd.vendor-v1+json').first }
+        .to raise_exception do |exception|
         expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
         expect(exception.headers).to eql({})
         expect(exception.status).to eql 406
-        expect(exception.message).to include('Accept header must not contain ranges ("*").')
-      end
-    end
-
-    it 'fails with 406 Not Acceptable if subtype is a range' do
-      expect { subject.call('HTTP_ACCEPT' => 'application/*').last }.to raise_exception do |exception|
-        expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
-        expect(exception.headers).to eql({})
-        expect(exception.status).to eql 406
-        expect(exception.message).to include('Accept header must not contain ranges ("*").')
+        expect(exception.message).to include('API vendor or version not found.')
       end
     end
 
@@ -270,10 +254,68 @@ describe Grape::Middleware::Versioner::Header do
 
     it 'fails with another version' do
       expect { subject.call('HTTP_ACCEPT' => 'application/vnd.vendor-v3+json') }.to raise_exception do |exception|
-        expect(exception).to be_a(Grape::Exceptions::InvalidAcceptHeader)
+        expect(exception).to be_a(Grape::Exceptions::InvalidVersionHeader)
         expect(exception.headers).to eql('X-Cascade' => 'pass')
         expect(exception.status).to eql 406
-        expect(exception.message).to include('API vendor or version not found')
+        expect(exception.message).to include('API version not found')
+      end
+    end
+  end
+
+  context 'when there are multiple versions with complex vendor specified with rescue_from :all' do
+    subject {
+      Class.new(Grape::API) do
+        rescue_from :all
+      end
+    }
+
+    let(:v1_app) {
+      Class.new(Grape::API) do
+        version 'v1', using: :header, vendor: 'test.a-cool_resource', cascade: false, strict: true
+        content_type :v1_test, 'application/vnd.test.a-cool_resource-v1+json'
+        formatter :v1_test, ->(object, _) { object }
+        format :v1_test
+
+        resources :users do
+          get :hello do
+            'one'
+          end
+        end
+      end
+    }
+
+    let(:v2_app) {
+      Class.new(Grape::API) do
+        version 'v2', using: :header, vendor: 'test.a-cool_resource', strict: true
+        content_type :v2_test, 'application/vnd.test.a-cool_resource-v2+json'
+        formatter :v2_test, ->(object, _) { object }
+        format :v2_test
+
+        resources :users do
+          get :hello do
+            'two'
+          end
+        end
+      end
+    }
+
+    def app
+      subject.mount v2_app
+      subject.mount v1_app
+      subject
+    end
+
+    context 'with header versioned endpoints and a rescue_all block defined' do
+      it 'responds correctly to a v1 request' do
+        versioned_get '/users/hello', 'v1', using: :header, vendor: 'test.a-cool_resource'
+        expect(last_response.body).to eq('one')
+        expect(last_response.body).not_to include('API vendor or version not found')
+      end
+
+      it 'responds correctly to a v2 request' do
+        versioned_get '/users/hello', 'v2', using: :header, vendor: 'test.a-cool_resource'
+        expect(last_response.body).to eq('two')
+        expect(last_response.body).not_to include('API vendor or version not found')
       end
     end
   end
