@@ -10,7 +10,7 @@ describe Grape::Validations::ParamsScope do
   end
 
   context 'setting a default' do
-    let(:documentation) { subject.routes.first.route_params }
+    let(:documentation) { subject.routes.first.params }
 
     context 'when the default value is truthy' do
       before do
@@ -67,7 +67,7 @@ describe Grape::Validations::ParamsScope do
     end
 
     it 'does not add documentation for the default value' do
-      documentation = subject.routes.first.route_params
+      documentation = subject.routes.first.params
       expect(documentation).to have_key('object')
       expect(documentation['object']).not_to have_key(:default)
     end
@@ -90,21 +90,24 @@ describe Grape::Validations::ParamsScope do
   end
 
   context 'when using custom types' do
-    class CustomType
-      attr_reader :value
-      def self.parse(value)
-        fail if value == 'invalid'
-        new(value)
-      end
+    module ParamsScopeSpec
+      class CustomType
+        attr_reader :value
 
-      def initialize(value)
-        @value = value
+        def self.parse(value)
+          fail if value == 'invalid'
+          new(value)
+        end
+
+        def initialize(value)
+          @value = value
+        end
       end
     end
 
     it 'coerces the parameter via the type\'s parse method' do
       subject.params do
-        requires :foo, type: CustomType
+        requires :foo, type: ParamsScopeSpec::CustomType
       end
       subject.get('/types') { params[:foo].value }
 
@@ -171,6 +174,24 @@ describe Grape::Validations::ParamsScope do
       end
     end
 
+    context 'when the default is an array' do
+      context 'and is the entire range of allowed values' do
+        it 'does not raise an exception' do
+          expect do
+            subject.params { optional :numbers, type: Array[Integer], values: 0..2, default: 0..2 }
+          end.to_not raise_error
+        end
+      end
+
+      context 'and is a subset of allowed values' do
+        it 'does not raise an exception' do
+          expect do
+            subject.params { optional :numbers, type: Array[Integer], values: [0, 1, 2], default: [1, 0] }
+          end.to_not raise_error
+        end
+      end
+    end
+
     context 'when both range endpoints are #kind_of? the type' do
       it 'accepts values in the range' do
         subject.params do
@@ -295,6 +316,32 @@ describe Grape::Validations::ParamsScope do
       expect(last_response.status).to eq(200)
     end
 
+    it 'applies the validations of multiple parameters' do
+      subject.params do
+        optional :a, :b
+        given :a, :b do
+          requires :c
+        end
+      end
+      subject.get('/multiple') { declared(params).to_json }
+
+      get '/multiple'
+      expect(last_response.status).to eq(200)
+
+      get '/multiple', a: true
+      expect(last_response.status).to eq(200)
+
+      get '/multiple', b: true
+      expect(last_response.status).to eq(200)
+
+      get '/multiple', a: true, b: true
+      expect(last_response.status).to eq(400)
+      expect(last_response.body).to eq('c is missing')
+
+      get '/multiple', a: true, b: true, c: true
+      expect(last_response.status).to eq(200)
+    end
+
     it 'raises an error if the dependent parameter was never specified' do
       expect do
         subject.params do
@@ -325,5 +372,37 @@ describe Grape::Validations::ParamsScope do
       expect(last_response.status).to eq(400)
       expect(last_response.body).to eq('bar[b] is missing')
     end
+
+    it 'includes the nested parameter within #declared(params)' do
+      subject.params do
+        requires :bar, type: Hash do
+          optional :a
+          given :a do
+            requires :b
+          end
+        end
+      end
+      subject.get('/nested') { declared(params).to_json }
+
+      get '/nested', bar: { a: true, b: 'yes' }
+      expect(JSON.parse(last_response.body)).to eq('bar' => { 'a' => 'true', 'b' => 'yes' })
+    end
+
+    it 'includes level 2 nested parameters outside the given within #declared(params)' do
+      subject.params do
+        requires :bar, type: Hash do
+          optional :a
+          given :a do
+            requires :c, type: Hash do
+              requires :b
+            end
+          end
+        end
+      end
+      subject.get('/nested') { declared(params).to_json }
+
+      get '/nested', bar: { a: true, c: { b: 'yes' } }
+      expect(JSON.parse(last_response.body)).to eq('bar' => { 'a' => 'true', 'c' => { 'b' => 'yes' } })
+    end
   end
 end
