@@ -318,6 +318,35 @@ describe Grape::Validations do
       end
     end
 
+    context 'hash with a required param with validation' do
+      before do
+        subject.params do
+          requires :items, type: Hash do
+            requires :key, type: String, values: %w(a b)
+          end
+        end
+        subject.get '/required' do
+          'required works'
+        end
+      end
+
+      it 'errors when param is not a Hash' do
+        get '/required', items: 'not a hash'
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('items is invalid, items[key] is missing, items[key] is invalid')
+
+        get '/required', items: [{ key: 'hash in array' }]
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('items is invalid, items[0][key] does not have a valid value')
+      end
+
+      it 'works when all params match' do
+        get '/required', items: { key: 'a' }
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('required works')
+      end
+    end
+
     context 'group' do
       before do
         subject.params do
@@ -337,7 +366,7 @@ describe Grape::Validations do
       end
 
       it "doesn't throw a missing param when param is present" do
-        get '/required', items: [key: 'hello', key: 'world']
+        get '/required', items: [key: 'hello']
         expect(last_response.status).to eq(200)
         expect(last_response.body).to eq('required works')
       end
@@ -374,10 +403,11 @@ describe Grape::Validations do
     end
 
     context 'custom validator for a Hash' do
-      module DateRangeValidations
-        class DateRangeValidator < Grape::Validations::Base
-          def validate_param!(attr_name, params)
-            unless params[attr_name][:from] <= params[attr_name][:to]
+      module ValuesSpec
+        module DateRangeValidations
+          class DateRangeValidator < Grape::Validations::Base
+            def validate_param!(attr_name, params)
+              return if params[attr_name][:from] <= params[attr_name][:to]
               fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: "'from' must be lower or equal to 'to'"
             end
           end
@@ -462,7 +492,7 @@ describe Grape::Validations do
         # NOTE: with body parameters in json or XML or similar this
         # should actually fail with: children[parents][name] is missing.
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children[parents] is missing')
+        expect(last_response.body).to eq('children[0][parents] is missing')
       end
 
       it 'safely handles empty arrays and blank parameters' do
@@ -473,7 +503,7 @@ describe Grape::Validations do
         expect(last_response.body).to eq('children is missing')
         get '/within_array', children: [name: 'Jay']
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children[parents] is missing')
+        expect(last_response.body).to eq('children[0][parents] is missing')
       end
 
       it 'errors when param is not an Array' do
@@ -488,7 +518,7 @@ describe Grape::Validations do
 
         get '/within_array', children: [name: 'Jay', parents: { name: 'Fred' }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children[parents] is invalid')
+        expect(last_response.body).to eq('children[0][parents] is invalid')
       end
     end
 
@@ -614,7 +644,7 @@ describe Grape::Validations do
           { name: 'Job', parents: [{ name: 'Joy' }] }
         ]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children[parents][name] is missing')
+        expect(last_response.body).to eq('children[0][parents][0][name] is missing')
       end
 
       it 'safely handles empty arrays and blank parameters' do
@@ -622,7 +652,7 @@ describe Grape::Validations do
         expect(last_response.status).to eq(200)
         put_with_json '/within_array', children: [name: 'Jay']
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children[parents] is missing')
+        expect(last_response.body).to eq('children[0][parents] is missing')
       end
     end
 
@@ -653,7 +683,7 @@ describe Grape::Validations do
       it 'errors when group is present, but required param is not' do
         get '/optional_group', items: [{ not_key: 'foo' }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items[key] is missing')
+        expect(last_response.body).to eq('items[0][key] is missing')
       end
 
       it "errors when param is present but isn't an Array" do
@@ -697,7 +727,7 @@ describe Grape::Validations do
       it 'does internal validations if the outer group is present' do
         get '/nested_optional_group', items: [{ key: 'foo' }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items[required_subitems] is missing')
+        expect(last_response.body).to eq('items[0][required_subitems] is missing')
 
         get '/nested_optional_group', items: [{ key: 'foo', required_subitems: [{ value: 'bar' }] }]
         expect(last_response.status).to eq(200)
@@ -707,7 +737,7 @@ describe Grape::Validations do
       it 'handles deep nesting' do
         get '/nested_optional_group', items: [{ key: 'foo', required_subitems: [{ value: 'bar' }], optional_subitems: [{ not_value: 'baz' }] }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items[optional_subitems][value] is missing')
+        expect(last_response.body).to eq('items[0][optional_subitems][0][value] is missing')
 
         get '/nested_optional_group', items: [{ key: 'foo', required_subitems: [{ value: 'bar' }], optional_subitems: [{ value: 'baz' }] }]
         expect(last_response.status).to eq(200)
@@ -717,7 +747,7 @@ describe Grape::Validations do
       it 'handles validation within arrays' do
         get '/nested_optional_group', items: [{ key: 'foo' }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items[required_subitems] is missing')
+        expect(last_response.body).to eq('items[0][required_subitems] is missing')
 
         get '/nested_optional_group', items: [{ key: 'foo', required_subitems: [{ value: 'bar' }] }]
         expect(last_response.status).to eq(200)
@@ -725,7 +755,7 @@ describe Grape::Validations do
 
         get '/nested_optional_group', items: [{ key: 'foo', required_subitems: [{ value: 'bar' }], optional_subitems: [{ not_value: 'baz' }] }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items[optional_subitems][value] is missing')
+        expect(last_response.body).to eq('items[0][optional_subitems][0][value] is missing')
       end
 
       it 'adds to declared parameters' do
@@ -763,9 +793,8 @@ describe Grape::Validations do
       module CustomValidations
         class Customvalidator < Grape::Validations::Base
           def validate_param!(attr_name, params)
-            unless params[attr_name] == 'im custom'
-              fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: 'is not custom!'
-            end
+            return if params[attr_name] == 'im custom'
+            fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: 'is not custom!'
           end
         end
       end
@@ -912,16 +941,15 @@ describe Grape::Validations do
         module CustomValidations
           class CustomvalidatorWithOptions < Grape::Validations::Base
             def validate_param!(attr_name, params)
-              unless params[attr_name] == @option[:text]
-                fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: @option[:error_message]
-              end
+              return if params[attr_name] == @option[:text]
+              fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message
             end
           end
         end
 
         before do
           subject.params do
-            optional :custom, customvalidator_with_options: { text: 'im custom with options', error_message: 'is not custom with options!' }
+            optional :custom, customvalidator_with_options: { text: 'im custom with options', message: 'is not custom with options!' }
           end
           subject.get '/optional_custom' do
             'optional with custom works!'
@@ -950,25 +978,26 @@ describe Grape::Validations do
         end
 
         it 'in helper module which kind of Grape::DSL::Helpers::BaseHelper' do
-          module SharedParams
+          shared_params = Module.new do
             extend Grape::DSL::Helpers::BaseHelper
             params :pagination do
             end
           end
-          subject.helpers SharedParams
+          subject.helpers shared_params
         end
       end
 
       context 'can be included in usual params' do
         before do
-          module SharedParams
+          shared_params = Module.new do
             extend Grape::DSL::Helpers::BaseHelper
             params :period do
               optional :start_date
               optional :end_date
             end
           end
-          subject.helpers SharedParams
+
+          subject.helpers shared_params
 
           subject.helpers do
             params :pagination do
@@ -1045,12 +1074,66 @@ describe Grape::Validations do
         subject.get '/' do
         end
 
-        expect(subject.routes.first.route_params['first_name'][:documentation]).to eq(documentation)
+        expect(subject.routes.first.params['first_name'][:documentation]).to eq(documentation)
+      end
+    end
+
+    context 'all or none' do
+      context 'optional params' do
+        before :each do
+          subject.resource :custom_message do
+            params do
+              optional :beer
+              optional :wine
+              optional :juice
+              all_or_none_of :beer, :wine, :juice, message: 'all params are required or none is required'
+            end
+            get '/all_or_none' do
+              'all_or_none works!'
+            end
+          end
+        end
+        context 'with a custom validation message' do
+          it 'errors when any one is present' do
+            get '/custom_message/all_or_none', beer: 'string'
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq 'beer, wine, juice all params are required or none is required'
+          end
+          it 'works when all params are present' do
+            get '/custom_message/all_or_none', beer: 'string', wine: 'anotherstring', juice: 'anotheranotherstring'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq 'all_or_none works!'
+          end
+          it 'works when none are present' do
+            get '/custom_message/all_or_none'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq 'all_or_none works!'
+          end
+        end
       end
     end
 
     context 'mutually exclusive' do
       context 'optional params' do
+        context 'with custom validation message' do
+          it 'errors when two or more are present' do
+            subject.resources :custom_message do
+              params do
+                optional :beer
+                optional :wine
+                optional :juice
+                mutually_exclusive :beer, :wine, :juice, message: 'are mutually exclusive cannot pass both params'
+              end
+              get '/mutually_exclusive' do
+                'mutually_exclusive works!'
+              end
+            end
+            get '/custom_message/mutually_exclusive', beer: 'string', wine: 'anotherstring'
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq 'beer, wine are mutually exclusive cannot pass both params'
+          end
+        end
+
         it 'errors when two or more are present' do
           subject.params do
             optional :beer
@@ -1069,6 +1152,34 @@ describe Grape::Validations do
       end
 
       context 'more than one set of mutually exclusive params' do
+        context 'with a custom validation message' do
+          it 'errors for all sets' do
+            subject.resources :custom_message do
+              params do
+                optional :beer
+                optional :wine
+                mutually_exclusive :beer, :wine, message: 'are mutually exclusive pass only one'
+                optional :nested, type: Hash do
+                  optional :scotch
+                  optional :aquavit
+                  mutually_exclusive :scotch, :aquavit, message: 'are mutually exclusive pass only one'
+                end
+                optional :nested2, type: Array do
+                  optional :scotch2
+                  optional :aquavit2
+                  mutually_exclusive :scotch2, :aquavit2, message: 'are mutually exclusive pass only one'
+                end
+              end
+              get '/mutually_exclusive' do
+                'mutually_exclusive works!'
+              end
+            end
+            get '/custom_message/mutually_exclusive', beer: 'true', wine: 'true', nested: { scotch: 'true', aquavit: 'true' }, nested2: [{ scotch2: 'true' }, { scotch2: 'true', aquavit2: 'true' }]
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq 'beer, wine are mutually exclusive pass only one, scotch, aquavit are mutually exclusive pass only one, scotch2, aquavit2 are mutually exclusive pass only one'
+          end
+        end
+
         it 'errors for all sets' do
           subject.params do
             optional :beer
@@ -1102,7 +1213,6 @@ describe Grape::Validations do
               optional :wine
               optional :beer
               optional :juice
-
               mutually_exclusive :beer, :wine, :juice
             end
           end
@@ -1156,6 +1266,18 @@ describe Grape::Validations do
     context 'exactly one of' do
       context 'params' do
         before :each do
+          subject.resources :custom_message do
+            params do
+              optional :beer
+              optional :wine
+              optional :juice
+              exactly_one_of :beer, :wine, :juice, message: { exactly_one: 'are missing, exactly one parameter is required', mutual_exclusion: 'are mutually exclusive, exactly one parameter is required' }
+            end
+            get '/exactly_one_of' do
+              'exactly_one_of works!'
+            end
+          end
+
           subject.params do
             optional :beer
             optional :wine
@@ -1167,6 +1289,26 @@ describe Grape::Validations do
           end
         end
 
+        context 'with a custom validation message' do
+          it 'errors when none are present' do
+            get '/custom_message/exactly_one_of'
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq 'beer, wine, juice are missing, exactly one parameter is required'
+          end
+
+          it 'succeeds when one is present' do
+            get '/custom_message/exactly_one_of', beer: 'string'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq 'exactly_one_of works!'
+          end
+
+          it 'errors when two or more are present' do
+            get '/custom_message/exactly_one_of', beer: 'string', wine: 'anotherstring'
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq 'beer, wine are mutually exclusive, exactly one parameter is required'
+          end
+        end
+
         it 'errors when none are present' do
           get '/exactly_one_of'
           expect(last_response.status).to eq(400)
@@ -1230,6 +1372,18 @@ describe Grape::Validations do
     context 'at least one of' do
       context 'params' do
         before :each do
+          subject.resources :custom_message do
+            params do
+              optional :beer
+              optional :wine
+              optional :juice
+              at_least_one_of :beer, :wine, :juice, message: 'are missing, please specify at least one param'
+            end
+            get '/at_least_one_of' do
+              'at_least_one_of works!'
+            end
+          end
+
           subject.params do
             optional :beer
             optional :wine
@@ -1241,6 +1395,26 @@ describe Grape::Validations do
           end
         end
 
+        context 'with a custom validation message' do
+          it 'errors when none are present' do
+            get '/custom_message/at_least_one_of'
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq 'beer, wine, juice are missing, please specify at least one param'
+          end
+
+          it 'does not error when one is present' do
+            get '/custom_message/at_least_one_of', beer: 'string'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq 'at_least_one_of works!'
+          end
+
+          it 'does not error when two are present' do
+            get '/custom_message/at_least_one_of', beer: 'string', wine: 'string'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq 'at_least_one_of works!'
+          end
+        end
+
         it 'errors when none are present' do
           get '/at_least_one_of'
           expect(last_response.status).to eq(400)
